/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2009 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 *
 *******************************************************************************
 *
 * $Id: IPTCVocabularyFactoryImpl.java,v d7e8fed5e750 2009/09/17 23:55:58 fabrizio $
 *
 ******************************************************************************/
package it.tidalwave.metadata.iptc.vocabularies.impl;

import java.util.List;
import java.util.zip.GZIPInputStream;
import java.io.InputStream;
import java.io.ObjectInputStream;
import it.tidalwave.metadata.iptc.vocabularies.IPTCVocabulary;
import it.tidalwave.metadata.iptc.vocabularies.IPTCVocabulary.Entry;
import it.tidalwave.metadata.iptc.vocabularies.IPTCVocabularyFactory;
import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: IPTCVocabularyFactoryImpl.java,v d7e8fed5e750 2009/09/17 23:55:58 fabrizio $
 *
 ******************************************************************************/
public class IPTCVocabularyFactoryImpl implements IPTCVocabularyFactory
  {
    @CheckForNull
    private IPTCVocabulary genreVocabulary;

    @CheckForNull
    private IPTCVocabulary providerVocabulary;

    @CheckForNull
    private IPTCVocabulary sceneVocabulary;

    @CheckForNull
    private IPTCVocabulary subjectCodeVocabulary;
    
    @edu.umd.cs.findbugs.annotations.SuppressWarnings("SIC_INNER_SHOULD_BE_STATIC_ANON")
    private IPTCVocabulary copyrightStatusVocabulary = new IPTCVocabulary()
      {
          {
            entries.add(null);
            entries.add("copyrighted");
            entries.add("not copyrighted");
            entriesInitialized = true;
          }
      };

    @Nonnull
    public synchronized IPTCVocabulary getGenreVocabulary() 
      {
        if (genreVocabulary == null)
          {
            genreVocabulary = loadVocabulary("topicset.iptc-genre.ser");
          }
        
        return genreVocabulary;
      }

    @Nonnull
    public synchronized IPTCVocabulary getProviderVocabulary() 
      {
        if (providerVocabulary == null)
          {
            providerVocabulary = loadVocabulary("topicset.iptc-provider.ser");
          }
        
        return providerVocabulary;
      }

    @Nonnull
    public synchronized IPTCVocabulary getSceneVocabulary() 
      {
        if (sceneVocabulary == null)
          {
            sceneVocabulary = loadVocabulary("topicset.iptc-scene.ser");
          }
        
        return sceneVocabulary;
      }

    @Nonnull
    public synchronized IPTCVocabulary getSubjectCodeVocabulary() 
      {
        if (subjectCodeVocabulary == null)
          {
            subjectCodeVocabulary = loadVocabulary("topicset.iptc-subjectcode.ser");
          }
        
        return subjectCodeVocabulary;
      }

    @Nonnull
    public IPTCVocabulary getCopyrightStatusVocabulary() 
      {
        return copyrightStatusVocabulary;
      }
    
    @Nonnull
    protected static IPTCVocabulary loadVocabulary (@Nonnull final String resourceName)
      {
        try
          {
            final IPTCVocabularyImpl vocabulary = new IPTCVocabularyImpl();
            final InputStream is = IPTCVocabularyFactoryImpl.class.getResourceAsStream("datasource/" + resourceName + ".gz");
            final ObjectInputStream ois = new ObjectInputStream(new GZIPInputStream(is));
            final List<Entry> entries = (List<Entry>)ois.readObject();
            vocabulary.internalGetEntries().addAll(entries);
            is.close();
            return vocabulary;
          }
        catch (RuntimeException e)
          {
            throw e;
          }
        catch (Exception e)
          {
            throw new RuntimeException(e);
          }
      }
  }
